package dev.fyloz.colorrecipesexplorer.utils

import dev.fyloz.colorrecipesexplorer.service.files.create
import java.io.File
import java.nio.file.Path

/** Mockable file wrapper, to prevent issues when mocking [java.io.File]. */
class WrappedFile(val file: File) {
    val isFile: Boolean
        get() = file.isFile

    fun toPath(): Path =
        file.toPath()

    fun exists() =
        file.exists()

    fun readBytes() =
        file.readBytes()

    fun writeBytes(array: ByteArray) =
        file.writeBytes(array)

    fun create() =
        file.create()

    fun delete(): Boolean =
        file.delete()

    companion object {
        fun from(path: String) =
            WrappedFile(File(path))

        fun from(path: FilePath) =
            from(path.path)
    }
}

@JvmInline
value class FilePath(val path: String)

/** Runs the given [block] in the context of a file with the given [fullPath]. */
fun <T> withFileAt(fullPath: FilePath, block: WrappedFile.() -> T) =
    WrappedFile.from(fullPath).block()